import json
import cmor
import unittest
import numpy
from pathlib import Path

from netCDF4 import Dataset

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = "TestTables/CMIP7_CV.json"

USER_INPUT = {
    "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
    "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
    "_cmip7_option": 1,
    "_controlled_vocabulary_file": CV_PATH,
    "activity_id": "CMIP",
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "drs_specs": "MIP-DRS7",
    "experiment_id": "piControl",
    "forcing_index": "f30",
    "grid_label": "gn",
    "initialization_index": "i000001d",
    "institution_id": "PCMDI",
    "license_id": "CC BY 4.0",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "physics_index": "p1",
    "realization_index": "r009",
    "source_id": "PCMDI-test-1-0",
    "tracking_prefix": "hdl:21.14100",
    "host_collection": "CMIP7",
    "frequency": "mon",
    "region": "glb",
    "archive_id": "WCRP",
    "output_path_template": "<activity_id><source_id><experiment_id><member_id><variable_id><branding_suffix><grid_label>",
    "output_file_template": "<variable_id><branding_suffix><frequency><region><grid_label><source_id><experiment_id><variant_label>",
}


class TestCMIP7(unittest.TestCase):

    def _load_dataset(self, updated_fields=None, removed_fields=None):
        input_data = USER_INPUT.copy()

        if removed_fields is not None:
            for field in removed_fields:
                input_data.pop(field, None)

        if updated_fields is not None:
            input_data.update(updated_fields)

        with open(self.input_json, "w") as input_file_handle:
            json.dump(input_data, input_file_handle, sort_keys=True, indent=4)

        error_flag = cmor.dataset_json(str(self.input_json))
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

    def setUp(self):
        """
        Write out a simple file using CMOR
        """
        self.input_json = Path("Test/input_cmip7.json")

        # Set up CMOR
        cmor.setup(inpath=CMIP7_TABLES_PATH, netcdf_file_action=cmor.CMOR_REPLACE)

        # Define dataset using USER_INPUT
        with open(self.input_json, "w") as input_file_handle:
            json.dump(USER_INPUT, input_file_handle, sort_keys=True, indent=4)

    def tearDown(self):
        self.input_json.unlink(missing_ok=True)

    def test_cmip7(self):
        self._load_dataset()

        data = [27] * (2 * 3 * 4)
        tos = numpy.array(data)
        tos.shape = (2, 3, 4)
        lat = numpy.array([10, 20, 30])
        lat_bnds = numpy.array([5, 15, 25, 35])
        lon = numpy.array([0, 90, 180, 270])
        lon_bnds = numpy.array([-45, 45,
                                135,
                                225,
                                315
                                ])
        time = numpy.array([15.5, 45])
        time_bnds = numpy.array([0, 31, 60])
        cmor.load_table("CMIP7_ocean.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             coord_vals=time,
                             cell_bounds=time_bnds,
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmortos = cmor.variable("tos_tavg-u-hxy-sea", "degC", axes)
        self.assertEqual(cmor.write(cmortos, tos), 0)
        filename = cmor.close(cmortos, file_name=True)
        self.assertEqual(cmor.close(), 0)

        ds = Dataset(filename)
        attrs = ds.ncattrs()
        test_attrs = {
            'branded_variable': 'tos_tavg-u-hxy-sea',
            'branding_suffix': 'tavg-u-hxy-sea',
            'temporal_label': 'tavg',
            'vertical_label': 'u',
            'horizontal_label': 'hxy',
            'area_label': 'sea',
            'region': 'glb',
            'frequency': 'mon',
            'archive_id': 'WCRP',
            'mip_era': 'CMIP7',
            'data_specs_version': 'CMIP-7.0.0.0',
            'host_collection': 'CMIP7',
        }

        for attr, val in test_attrs.items():
            self.assertIn(attr, attrs)
            self.assertEqual(val, ds.getncattr(attr))
        institution_id = USER_INPUT["institution_id"]
        license_id = "CC BY 4.0"
        license_type = "Creative Commons Attribution 4.0 International"
        license_url = "https://creativecommons.org/licenses/by/4.0/"
        license = \
            (f"{license_id}; CMIP7 data produced by {institution_id} is "
             f"licensed under a {license_type} License ({license_url}). "
             "Consult https://wcrp-cmip.github.io/cmip7-guidance/docs/CMIP7/"
             "Guidance_for_users for terms of "
             "use governing CMIP7 output, including citation requirements and "
             "proper acknowledgment. The data producers and data providers "
             "make no warranty, either express or implied, including, but not "
             "limited to, warranties of merchantability and fitness for a "
             "particular purpose. All liabilities arising from the supply of "
             "the information (including any liability arising in negligence) "
             "are excluded to the fullest extent permitted by law."
             )
        self.assertTrue("license" in attrs)
        self.assertTrue("license_id" in attrs)
        self.assertTrue("license_type" not in attrs)
        self.assertTrue("license_url" not in attrs)
        self.assertEqual(license, ds.getncattr("license"))
        self.assertEqual(license_id, ds.getncattr("license_id"))

        ds.close()

    def test_secondary_modeling_realm(self):
        self._load_dataset()

        data = [27] * (2 * 3 * 4)
        pr = numpy.array(data)
        pr.shape = (2, 3, 4)
        lat = numpy.array([10, 20, 30])
        lat_bnds = numpy.array([5, 15, 25, 35])
        lon = numpy.array([0, 90, 180, 270])
        lon_bnds = numpy.array([-45, 45,
                                135,
                                225,
                                315
                                ])
        time = numpy.array([15.5, 45])
        time_bnds = numpy.array([0, 31, 60])
        cmor.load_table("CMIP7_atmos.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             coord_vals=time,
                             cell_bounds=time_bnds,
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmorpr = cmor.variable("prra_tavg-u-hxy-is", "kg m-2 s-1", axes)
        self.assertEqual(cmor.write(cmorpr, pr), 0)
        filename = cmor.close(cmorpr, file_name=True)
        self.assertEqual(cmor.close(), 0)

        ds = Dataset(filename)
        attrs = ds.ncattrs()
        test_attrs = {
            'branded_variable': 'prra_tavg-u-hxy-is',
            'branding_suffix': 'tavg-u-hxy-is',
            'temporal_label': 'tavg',
            'vertical_label': 'u',
            'horizontal_label': 'hxy',
            'area_label': 'is',
            'region': 'glb',
            'frequency': 'mon',
            'archive_id': 'WCRP',
            'mip_era': 'CMIP7',
            'data_specs_version': 'CMIP-7.0.0.0',
            'host_collection': 'CMIP7',
            'realm': 'atmos land landIce',
        }

        for attr, val in test_attrs.items():
            self.assertIn(attr, attrs)
            self.assertEqual(val, ds.getncattr(attr))

        ds.close()

    def test_climatology(self):
        self._load_dataset()

        data = numpy.array([27, 28])
        time = numpy.array([15, 45])
        time_bnds = numpy.array([[0, 31], [31, 60]])
        cmor.load_table("CMIP7_atmos.json")
        cmortime = cmor.axis("time2",
                             coord_vals=time,
                             cell_bounds=time_bnds,
                             units="days since 2018")
        cmorco2 = cmor.variable("co2_tclm-u-hm-u", "mol mol-1", [cmortime])
        self.assertEqual(cmor.write(cmorco2, data), 0)
        filename = cmor.close(cmorco2, file_name=True)
        self.assertEqual(cmor.close(), 0)

        basename = Path(filename).name
        self.assertTrue(basename.endswith("_201801-201802.nc"))

        ds = Dataset(filename)
        self.assertEqual("mon", ds.getncattr("frequency"))
        time_var = ds.variables["time"]
        self.assertEqual("climatology_bnds", time_var.getncattr("climatology"))
        self.assertEqual("Monthly Climatology", time_var.getncattr("long_name"))

        ds.close()

    def test_conventions_set_by_user(self):
        self._load_dataset(updated_fields={"Conventions": "CF-1.13"})

        data = numpy.array([27, 28])
        time = numpy.array([15, 45])
        time_bnds = numpy.array([[0, 31], [31, 60]])
        cmor.load_table("CMIP7_atmos.json")
        cmortime = cmor.axis("time2",
                             coord_vals=time,
                             cell_bounds=time_bnds,
                             units="days since 2018")
        cmorco2 = cmor.variable("co2_tclm-u-hm-u", "mol mol-1", [cmortime])
        self.assertEqual(cmor.write(cmorco2, data), 0)
        filename = cmor.close(cmorco2, file_name=True)
        self.assertEqual(cmor.close(), 0)

        ds = Dataset(filename)
        self.assertEqual("CF-1.13", ds.getncattr("Conventions"))

        ds.close()


if __name__ == '__main__':
    unittest.main()



import json
import shutil
import tempfile
import unittest
from pathlib import Path

import cmor
import numpy
from netCDF4 import Dataset

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = Path("TestTables/CMIP7_CV.json")

USER_INPUT = {
    "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
    "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
    "_cmip7_option": 1,
    "_controlled_vocabulary_file": None,
    "activity_id": "CMIP",
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "experiment_id": "piControl",
    "forcing_index": "f30",
    "grid_label": "gn",
    "initialization_index": "i000001d",
    "institution_id": "PCMDI",
    "license_id": "CC BY 4.0",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "physics_index": "p1",
    "realization_index": "r009",
    "source_id": "PCMDI-test-1-0",
    "host_collection": "CMIP7",
    "frequency": "mon",
    "region": "glb",
    "archive_id": "WCRP",
}


class TestCVRootStringAttributes(unittest.TestCase):
    def setUp(self):
        self.tmpdir = Path(tempfile.mkdtemp(dir=str(Path("Test").resolve())))
        self.cv_path = self.tmpdir / "CMIP7_CV_root_strings.json"
        self.input_path = self.tmpdir / "input.json"
        self.output_dir = self.tmpdir / "output"
        self.output_dir.mkdir()
        self.cv_relpath = self.cv_path.relative_to(Path.cwd())
        self.input_relpath = self.input_path.relative_to(Path.cwd())
        self.output_relpath = self.output_dir.relative_to(Path.cwd())

    def tearDown(self):
        shutil.rmtree(self.tmpdir, ignore_errors=True)

    def test_sets_drs_specs_and_tracking_prefix_from_cv_root_strings(self):
        with CV_PATH.open() as cv_infile:
            cv = json.load(cv_infile)

        cv["CV"]["drs_specs"] = "MIP-DRS7"
        cv["CV"]["tracking_prefix"] = "hdl:21.14100"

        with self.cv_path.open("w") as cv_outfile:
            json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        user_input = USER_INPUT.copy()
        user_input["_controlled_vocabulary_file"] = str(self.cv_relpath)
        user_input["outpath"] = str(self.output_relpath)

        with self.input_path.open("w") as input_file:
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        cmor.setup(inpath=CMIP7_TABLES_PATH, netcdf_file_action=cmor.CMOR_REPLACE)
        error_flag = cmor.dataset_json(str(self.input_relpath))
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        data = numpy.array([27] * (2 * 3 * 4))
        data.shape = (2, 3, 4)
        lat = numpy.array([10, 20, 30])
        lat_bnds = numpy.array([5, 15, 25, 35])
        lon = numpy.array([0, 90, 180, 270])
        lon_bnds = numpy.array([-45, 45, 135, 225, 315])
        time = numpy.array([15.5, 45])
        time_bnds = numpy.array([0, 31, 60])

        cmor.load_table("CMIP7_ocean.json")
        cmorlat = cmor.axis(
            "latitude",
            coord_vals=lat,
            cell_bounds=lat_bnds,
            units="degrees_north",
        )
        cmorlon = cmor.axis(
            "longitude",
            coord_vals=lon,
            cell_bounds=lon_bnds,
            units="degrees_east",
        )
        cmortime = cmor.axis(
            "time",
            coord_vals=time,
            cell_bounds=time_bnds,
            units="days since 2018",
        )
        cmortos = cmor.variable("tos_tavg-u-hxy-sea", "degC", [cmortime, cmorlat, cmorlon])
        self.assertEqual(cmor.write(cmortos, data), 0)

        filename = Path(cmor.close(cmortos, file_name=True))
        self.assertEqual(cmor.close(), 0)

        ds = Dataset(filename)
        self.assertNotIn("tracking_prefix", ds.ncattrs())
        self.assertEqual(ds.getncattr("drs_specs"), "MIP-DRS7")
        self.assertTrue(ds.getncattr("tracking_id").startswith("hdl:21.14100/"))
        ds.close()

        filename.unlink(missing_ok=True)


if __name__ == "__main__":
    unittest.main()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.
import cmor
import base_CMIP6_CV
import sys
import os
import tempfile
import json
import signal
import unittest

def run():
    unittest.main()

class TestdirectoryMethods(base_CMIP6_CV.BaseCVsTest):

    def bad_outpath(self):
        override = os.environ.get("CMOR_BAD_DIRECTORY_OUTPATH")
        if override:
            return override

        # cibuildwheel Linux tests run as root inside the manylinux container,
        # so the historical "/CMIP6" path is writable there.
        if hasattr(os, "geteuid") and os.geteuid() == 0 and sys.platform.startswith("linux"):
            return "/proc/CMIP6"

        return "/CMIP6"

    def test_Directory(self):

        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        with open("Test/baddirectory.json") as source:
            dataset = json.load(source)
        dataset["outpath"] = self.bad_outpath()
        with tempfile.NamedTemporaryFile(mode="w", suffix=".json", delete=False) as tmp_json:
            json.dump(dataset, tmp_json)
            tmp_json_path = tmp_json.name
        self.addCleanup(lambda: os.path.exists(tmp_json_path) and os.unlink(tmp_json_path))
        try:
            cmor.dataset_json(tmp_json_path)
        except BaseException:
            pass
        self.assertCV("unable to create this directory")

if __name__ == '__main__':
    run()



import json
import cmor
import unittest
import numpy
from pathlib import Path

from base_CMIP6_CV import BaseCVsTest
from netCDF4 import Dataset

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = "TestTables/CMIP7_CV.json"

USER_INPUT = {
    "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
    "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
    "_cmip7_option": 1,
    "_controlled_vocabulary_file": CV_PATH,
    "activity_id": "CMIP",
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "drs_specs": "MIP-DRS7",
    "experiment_id": "piControl",
    "forcing_index": "f30",
    "grid_label": "gn",
    "initialization_index": "i000001d",
    "institution_id": "PCMDI",
    "license_id": "CC BY 4.0",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "physics_index": "p1",
    "realization_index": "r009",
    "source_id": "PCMDI-test-1-0",
    "tracking_prefix": "hdl:21.14100",
    "host_collection": "CMIP7",
    "frequency": "mon",
    "region": "glb",
    "archive_id": "WCRP",
    "output_path_template": "<activity_id><source_id><experiment_id><member_id><variable_id><branding_suffix><grid_label>",
    "output_file_template": "<variable_id><branding_suffix><frequency><region><grid_label><source_id><experiment_id><variant_label>",
}


class TestCMIP7WithParentAttributes(BaseCVsTest):
    def setUp(self):
        """
        Write out a simple file using CMOR
        """
        super().setUp()
        self.input_json = Path("Test/input_parent_attrs.json")

        # Set up CMOR
        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

    def tearDown(self):
        self.input_json.unlink(missing_ok=True)
        super().tearDown()

    def _load_dataset(self, overrides=None, removed_fields=None):
        input_data = USER_INPUT.copy()

        if removed_fields is not None:
            for field in removed_fields:
                input_data.pop(field, None)

        if overrides is not None:
            input_data.update(overrides)

        with open(self.input_json, "w") as input_file_handle:
            json.dump(input_data, input_file_handle, sort_keys=True, indent=4)

        error_flag = cmor.dataset_json(str(self.input_json))
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

    def _write_tos_file(self):
        data = [27] * (2 * 3 * 4)
        tos = numpy.array(data)
        tos.shape = (2, 3, 4)
        lat = numpy.array([10, 20, 30])
        lat_bnds = numpy.array([5, 15, 25, 35])
        lon = numpy.array([0, 90, 180, 270])
        lon_bnds = numpy.array([-45, 45,
                                135,
                                225,
                                315
                                ])
        time = numpy.array([15.5, 45.5])
        time_bnds = numpy.array([0, 31, 60])
        cmor.load_table("CMIP7_ocean.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             coord_vals=time,
                             cell_bounds=time_bnds,
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmortos = cmor.variable("tos_tavg-u-hxy-sea", "degC", axes)
        self.assertEqual(cmor.write(cmortos, tos), 0)
        filename = cmor.close(cmortos, file_name=True)
        self.delete_files.append(filename)
        self.assertEqual(cmor.close(), 0)
        return filename

    def test_cmip7_without_parent_attributes(self):
        self._load_dataset()
        filename = self._write_tos_file()

        ds = Dataset(filename)
        attrs = ds.ncattrs()

        parent_attrs = [
            "branch_time_in_child",
            "branch_time_in_parent",
            "parent_mip_era",
            "parent_time_units",
            "parent_activity_id",
            "parent_source_id",
            "parent_experiment_id",
            "parent_variant_label"
        ]

        for attr in parent_attrs:
            self.assertNotIn(attr, attrs)

        ds.close()

    def test_cmip7_with_parent_attributes(self):
        self._load_dataset(
            overrides={
                "branch_time_in_child": 30.0,
                "branch_time_in_parent": 10800.0,
                "experiment_id": "historical",
                "parent_mip_era": "CMIP7",
                "parent_time_units": "days since 1850-01-01",
                "parent_source_id": "PCMDI-test-1-0",
                "parent_experiment_id": "piControl",
                "parent_activity_id": "CMIP",
                "parent_variant_label": "r1i1p1f3",
            },
        )
        filename = self._write_tos_file()

        ds = Dataset(filename)
        attrs = ds.ncattrs()

        parent_attrs = {
            "branch_time_in_child": 30.0,
            "branch_time_in_parent": 10800.0,
            "parent_mip_era": "CMIP7",
            "parent_time_units": "days since 1850-01-01",
            "parent_activity_id": "CMIP",
            "parent_source_id": "PCMDI-test-1-0",
            "parent_experiment_id": "piControl",
            "parent_variant_label": "r1i1p1f3"
        }

        for attr, value in parent_attrs.items():
            self.assertIn(attr, attrs)
            self.assertEqual(value, ds.getncattr(attr))

        ds.close()

    def test_cmip7_warns_on_parent_attributes_without_parent_experiment(self):
        self._load_dataset(
            overrides={
                "branch_time_in_child": 30.0,
                "parent_activity_id": "CMIP",
            },
        )
        filename = self._write_tos_file()

        with Dataset(filename) as ds:
            self.assertNotIn("branch_time_in_child", ds.ncattrs())
            self.assertNotIn("parent_activity_id", ds.ncattrs())

        log_contents = Path(self.tmpfile).read_text()
        self.assertIn(
            'Warning: Your experiment does not have a "parent_experiment_id" '
            'defined but your dataset has a "branch_time_in_child" defined.',
            log_contents,
        )
        self.assertIn(
            'The "branch_time_in_child" will be removed from your dataset.',
            log_contents,
        )
        self.assertIn(
            'Warning: Your experiment does not have a "parent_experiment_id" '
            'defined but your dataset has a "parent_activity_id" defined.',
            log_contents,
        )
        self.assertIn(
            'The "parent_activity_id" will be removed from your dataset.',
            log_contents,
        )

    def test_cmip7_errors_on_parent_experiment_without_cv_parent(self):
        self._load_dataset(
            overrides={
                "parent_experiment_id": "historical",
            },
        )

        with self.assertRaises(cmor.CMORError):
            self._write_tos_file()

        try:
            cmor.close()
        except BaseException:
            pass

        self.assertCV(
            'Your experiment "piControl" does not have parent experiments.',
            'Error:',
            number_of_lines_to_scan=6,
        )
        self.assertCV(
            'should not define "parent_experiment_id" in their datasets.',
            'Error:',
            number_of_lines_to_scan=6,
        )

    def test_cmip7_errors_when_parent_experiment_required_but_missing(self):
        self._load_dataset(
            overrides={
                "experiment_id": "historical",
            },
        )

        with self.assertRaises(cmor.CMORError):
            self._write_tos_file()

        try:
            cmor.close()
        except BaseException:
            pass

        self.assertCV(
            'Your input attribute "parent_experiment_id" is not defined properly',
            'Error:',
            number_of_lines_to_scan=6,
        )
        self.assertCV(
            'for your experiment "historical"',
            'Error:',
            number_of_lines_to_scan=6,
        )

    def test_cmip7_errors_on_parent_experiment_not_in_cv(self):
        self._load_dataset(
            overrides={
                "branch_time_in_child": 30.0,
                "branch_time_in_parent": 10800.0,
                "experiment_id": "historical",
                "parent_mip_era": "CMIP7",
                "parent_time_units": "days since 1850-01-01",
                "parent_source_id": "PCMDI-test-1-0",
                "parent_experiment_id": "badParent",
                "parent_activity_id": "CMIP",
                "parent_variant_label": "r1i1p1f3",
            },
        )

        with self.assertRaises(cmor.CMORError):
            self._write_tos_file()

        try:
            cmor.close()
        except BaseException:
            pass

        self.assertCV(
            'Your input attribute "parent_experiment_id" with value',
            'Error:',
            number_of_lines_to_scan=8,
        )
        self.assertCV(
            '"badParent" needs to be replaced with value "piControl"',
            'Error:',
            number_of_lines_to_scan=8,
        )

    def test_cmip7_errors_on_incorrect_parent_activity_id(self):
        self._load_dataset(
            overrides={
                "branch_time_in_child": 30.0,
                "branch_time_in_parent": 10800.0,
                "experiment_id": "historical",
                "parent_mip_era": "CMIP7",
                "parent_time_units": "days since 1850-01-01",
                "parent_source_id": "PCMDI-test-1-0",
                "parent_experiment_id": "piControl",
                "parent_activity_id": "BadMIP",
                "parent_variant_label": "r1i1p1f3",
            },
        )

        with self.assertRaises(cmor.CMORError):
            self._write_tos_file()

        try:
            cmor.close()
        except BaseException:
            pass

        self.assertCV(
            'Your input attribute "parent_activity_id" with value',
            'Error:',
            number_of_lines_to_scan=8,
        )
        self.assertCV(
            '"BadMIP" needs to be replaced with value "CMIP"',
            'Error:',
            number_of_lines_to_scan=8,
        )


if __name__ == '__main__':
    unittest.main()



import os
import sys

prefix_xml_pth = os.path.join(sys.prefix, "share", "udunits", "udunits2.xml")
wheel_xml_pth = os.path.join(os.path.dirname(__file__), "data", "udunits2.xml")

if not os.environ.get("UDUNITS2_XML_PATH"):
    if os.path.exists(wheel_xml_pth):
        os.environ["UDUNITS2_XML_PATH"] = wheel_xml_pth
    elif os.path.exists(prefix_xml_pth):
        os.environ["UDUNITS2_XML_PATH"] = prefix_xml_pth

from cmor.cmor_const import *

from cmor.pywrapper import (
    CMORError, axis, variable, write, setup, load_table, set_table, zfactor,
    close, grid, set_grid_mapping, set_crs,
    dataset_json, set_cur_dataset_attribute, get_cur_dataset_attribute,
    has_cur_dataset_attribute, set_variable_attribute, get_variable_attribute,
    has_variable_attribute, get_final_filename, set_deflate, set_zstandard,
    set_quantize, set_furtherinfourl, set_climatology, get_climatology,
    set_terminate_signal, get_terminate_signal, set_chunking)

try:
    from check_CMOR_compliant import checkCMOR
except ImportError:
    pass
